﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using ePlanner.JobShop;

namespace ePlanner.BCO
{
    class Hive
    {
        static Random random = null; 

        public Graph graphPlan; //  data na optimalizaci 
        public Graph emptyGraph; // graf bez disjunktivnich hran 
        public int totalNumberBees; 
        public int numberInactive;
        public int numberActive;
        public int numberScout;

        public int maxNumberCycles; 
        public int maxNumberVisits; // maximalni pocet navstev nez dojde k opusteni reseni
        public double probPersuasion = 0.90; // pravdepodobnost, ze vcela bude presvedecena waggle dancem prijmout lepsi reseni
        public double probMistake = 0.01; // pravdepodobnost, ze vcela udela chybu v prijimani reseni

        public Bee[] bees;
        public Graph bestMemoryMatrix; // nejlepsi reseni
        public double bestMeasureOfQuality;
        public int[] indexesOfInactiveBees;  // pole neaktivnich vcel
   
        public Hive(int totalNumberBees, int numberInactive, int numberActive, int numberScout, int maxNumberVisits,
          int maxNumberCycles, Graph basicGraphPlan, Graph emptyGraph)
        {
            random = new Random(0);

            this.totalNumberBees = totalNumberBees;
            this.numberInactive = numberInactive;
            this.numberActive = numberActive;
            this.numberScout = numberScout;
            this.maxNumberVisits = maxNumberVisits;
            this.maxNumberCycles = maxNumberCycles;

            this.graphPlan = basicGraphPlan;
            this.emptyGraph = emptyGraph;

            this.bees = new Bee[totalNumberBees];

            this.bestMemoryMatrix = graphPlan;
            this.bestMeasureOfQuality = this.bestMemoryMatrix.MakeSpan;

            this.indexesOfInactiveBees = new int[numberInactive]; 

            for (int i = 0; i < totalNumberBees; ++i) //inicializace vcel a jejich reseni
            {
                // rozdeleni vcel do skupin
                int currStatus; 
                if (i < numberInactive)
                {
                    currStatus = 0; // inactive
                    indexesOfInactiveBees[i] = i; 
                }
                else if (i < numberInactive + numberScout)
                {
                    currStatus = 2; // scout
                }
                else
                {
                    currStatus = 1; // active
                }
                
                Graph localCopyOfEmptyGraph = new Graph();
                localCopyOfEmptyGraph = Extensions.DeepClone(this.emptyGraph);
                Graph randomMemoryMatrix = GenerateRandomMemoryMatrix(localCopyOfEmptyGraph);
                double mq = randomMemoryMatrix.MakeSpan;
                int numberOfVisits = 0;

                bees[i] = new Bee(currStatus, randomMemoryMatrix, mq, numberOfVisits); 

                // ma tato vcela nejlepsi reseni?
                if (bees[i].measureOfQuality < bestMeasureOfQuality) // mensi hodnota je lepsi
                {
                    this.bestMemoryMatrix = bees[i].memoryMatrix;
                    this.bestMeasureOfQuality = bees[i].measureOfQuality;
                }
            } // bee

        } // ctor


        // pocatecni konfigurace
        public Graph GenerateRandomMemoryMatrix(Graph emptyGraph)
        {
            try
            {
                emptyGraph.machines.ForEach(m => emptyGraph.AddDisjunctiveEdges(m));
                emptyGraph.ComputeCPM();

                return emptyGraph;

            }
            catch (Exception ex)
            {
                throw ex;
            }
        } // GenerateRandomMemoryMatrix()

        // generovani sousedniho reseni
        public Graph GenerateNeighborMemoryMatrix(Graph memoryMatrix)
        {
            try
            {
                List<Machine> machinesWithCritOps = memoryMatrix.machines.Where(m => m.operations.Where(o => o.IsCritical).ToList().Count > 2).ToList();

                Random rnd = new Random();
                int r = rnd.Next(0, machinesWithCritOps.Count);
                List<Operation> randomChoosedMachine = machinesWithCritOps[r].operations;

                for (int i = 1; i < randomChoosedMachine.Count - 1; i++)
                {
                    Operation criticalOP = randomChoosedMachine[i];
                    if (criticalOP.IsCritical)
                    {
                        Operation prevCritOp = randomChoosedMachine[i - 1];
                        if (prevCritOp.IsCritical)
                        {
                            int order = prevCritOp.OrderInMachine;
                            prevCritOp.OrderInMachine = criticalOP.OrderInMachine;
                            criticalOP.OrderInMachine = order;
                            
                            prevCritOp.Succ.Remove(criticalOP);
                            prevCritOp.Pred.Add(criticalOP);

                            criticalOP.Pred.Remove(prevCritOp);
                            criticalOP.Succ.Add(prevCritOp);

                            bool isAcyclic = memoryMatrix.IsAcyclic();
                            if (!isAcyclic)
                            {
                                criticalOP.Succ.Remove(prevCritOp);
                                criticalOP.Pred.Add(prevCritOp);

                                prevCritOp.Pred.Remove(criticalOP);
                                prevCritOp.Succ.Add(criticalOP);
                            }
                            else
                            {
                                break;
                            }
                        }
                    }
                }
                memoryMatrix.ComputeCPM();
                return memoryMatrix;

            }
            catch (Exception ex)
            {
                throw ex;
            }

        } // GenerateNeighborMemoryMatrix()


        // MakeSpan
        public double MeasureOfQuality(List<Operation> memoryMatrix)
        {
            try
            {
                double answer = Convert.ToDouble(memoryMatrix[memoryMatrix.Count - 1].Let);
 
                return answer;
            }
            catch (Exception ex)
            {
                throw ex;
            }

        } // MeasureOfQuality()

        // proces hledani nejlepsiho reseni
        public void Solve(bool doProgressBar) 
        { 
            int numberOfSymbolsToPrint = 10; // postup po 10 %
            int increment = this.maxNumberCycles / numberOfSymbolsToPrint;
            if (doProgressBar) System.Diagnostics.Debug.WriteLine("Progress: |==========|"); 
            if (doProgressBar) System.Diagnostics.Debug.Write("           ");
            int cycle = 0;

            while (cycle < this.maxNumberCycles)
            {
                for (int i = 0; i < this.totalNumberBees; ++i)
                {
                    if (this.bees[i].status == 1) // active bee
                        ProcessActiveBee(i);
                    else if (this.bees[i].status == 2) // scout bee
                        ProcessScoutBee(i);
                    else if (this.bees[i].status == 0) // inactive bee
                        ProcessInactiveBee(i);
                } 
                ++cycle;

                if (doProgressBar && cycle % increment == 0)
                    System.Diagnostics.Debug.Write("^");
            } //end while

            if (doProgressBar) System.Diagnostics.Debug.WriteLine(""); // konec progres baru
        } // Solve()

        private void ProcessInactiveBee(int i)
        {
            return; // nedelej nic
        }

        private void ProcessActiveBee(int i)
        {
            try
            {
                Graph copyOfLocalSolution = new Graph();
                copyOfLocalSolution = Extensions.DeepClone(bees[i].memoryMatrix);
                Graph neighbor = GenerateNeighborMemoryMatrix(copyOfLocalSolution); // najdi sousedni reseni
                double neighborQuality = neighbor.MakeSpan; 
                double prob = random.NextDouble(); // slouzi k porovnani jestli udelala vcela chybu - mala sence - mensi nez ~0.01
                bool memoryWasUpdated = false; // priznak => if true->vcela do waggle dance
                bool numberOfVisitsOverLimit = false; //  priznak => if true->vcela se stane inactive

                if (neighborQuality < bees[i].measureOfQuality) // sousedni reseni je lepsi nez aktualni
                {
                    if (prob < probMistake) // vcela udela chybu - odmitne lepsi reseni
                    {
                        ++bees[i].numberOfVisits; 
                        if (bees[i].numberOfVisits > maxNumberVisits) 
                            numberOfVisitsOverLimit = true;
                    }
                    else // vcela neudela chybu - prijme lepsi reseni
                    {
                        bees[i].memoryMatrix = neighbor;
                        bees[i].measureOfQuality = neighborQuality;
                        bees[i].numberOfVisits = 0; // zmena reseni => vynulovani pocitadla
                        memoryWasUpdated = true; // bude delat waggle dance 
                    }
                }
                else // aktualni reseni je lepsi nez nove nalezene
                {
                    if (prob < probMistake)// vcela udela chybu - prijme horsi reseni
                    {
                        bees[i].memoryMatrix = Extensions.DeepClone(neighbor);
                        bees[i].measureOfQuality = neighborQuality;
                        bees[i].numberOfVisits = 0; // zmena reseni => vynulovani pocitadla
                        memoryWasUpdated = true;  // bude delat waggle dance 
                    }
                    else  // vcela neudela chybu - odmitne horsireseni
                    {
                        ++bees[i].numberOfVisits;
                        if (bees[i].numberOfVisits > maxNumberVisits) numberOfVisitsOverLimit = true;
                    }
                }

                // kdyz je aktualni reseni vycerpane, vcela se stane inaktivni
                if (numberOfVisitsOverLimit == true)
                {
                    bees[i].status = 0; // zmena typu vcely
                    bees[i].numberOfVisits = 0; // reset pocitadla
                    int x = random.Next(numberInactive); // vyber nahodne neaktivni vcely
                    bees[indexesOfInactiveBees[x]].status = 1; // nahodna neaktivni cela se stane aktivni vcelou
                    indexesOfInactiveBees[x] = i; // soucasna vcela se ulozi do pole neaktivnich vcel
                }
                else if (memoryWasUpdated == true) // vcela nasla nejake lepsi reseni a bude presvedcovat ostatni vcela waggle tancem
                {
                    if (bees[i].measureOfQuality < this.bestMeasureOfQuality) // nove reseni je lepsi nez globalni reseni
                    {
                        this.bestMemoryMatrix = Extensions.DeepClone(bees[i].memoryMatrix);
                        this.bestMeasureOfQuality = bees[i].measureOfQuality; // zmena globalniho reseni
                    }
                    DoWaggleDance(i);
                }
                else // nic se nestalo, vcela se vraci do uluhu (a ceka na dalsi iteraci alg.)
                {
                    return;
                }

            }
            catch (Exception ex)
            {
                throw ex;
            }

        } // ProcessActiveBee()

        
        private void ProcessScoutBee(int i)
        {
            try
            {
                Graph copyOfEmptyGraph = new Graph();
                copyOfEmptyGraph = Extensions.DeepClone(emptyGraph);

                Graph randomFoodSource = GenerateRandomMemoryMatrix(copyOfEmptyGraph); // vcela scout hleda nahodne reseni
                
                double randomFoodSourceQuality = randomFoodSource.MakeSpan; 
                if (randomFoodSourceQuality < bees[i].measureOfQuality) // scout bee nasla lepsi reseni nez soucasne
                {
                    bees[i].memoryMatrix = randomFoodSource;
                    bees[i].measureOfQuality = randomFoodSourceQuality;

                    // je toto reseni lepsi nez globalni nejlepsi?
                    if (bees[i].measureOfQuality < bestMeasureOfQuality) // mensi => lepsi
                    {
                        this.bestMemoryMatrix = bees[i].memoryMatrix;
                        this.bestMeasureOfQuality = bees[i].measureOfQuality;
                    } 

                    DoWaggleDance(i); // scout se vraci do ulu delat waggle dance

                } 

            }
            catch (Exception ex)
            {
                throw ex;
            }
        } // ProcessScoutBee()

        private void DoWaggleDance(int i)
        {
            try
            {
                for (int ii = 0; ii < numberInactive; ++ii) 
                {
                    int b = indexesOfInactiveBees[ii]; 
                    if (bees[b].status != 0) 
                        throw new Exception("Catastrophic logic error when scout bee waggles dances");
                    if (bees[b].numberOfVisits != 0) 
                        throw new Exception("Found an inactive bee with numberOfVisits != 0 in Scout bee waggle dance routine");
                    if (bees[i].measureOfQuality < bees[b].measureOfQuality) // scout bee nasla lepsi reseni nez maji vcely co ji prihlizi
                    {
                        double p = random.NextDouble(); 
                        if (this.probPersuasion > p) // inaktivni vcela je presvedcena scoutem
                        {
                            bees[b].memoryMatrix = Extensions.DeepClone(bees[i].memoryMatrix);
                            bees[b].measureOfQuality = bees[i].measureOfQuality;
                        } 
                    } 
                } 
            }
            catch (Exception ex)
            {
                throw ex;
            }
        } // DoWaggleDance()

    } // class

 
}