﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ePlanner.JobShop
{
    [Serializable()]
    public class Graph
    {
        List<Job> jobs;
        public List<Machine> machines;
        public List<Operation> operations;
        public List<Operation> criticalOperations = new List<Operation>();
        Operation Source;
        Operation Sink;
        public float MakeSpan { get; set; }

        public Graph()
        {
            this.jobs = new List<Job>();
            this.machines = new List<Machine>();

            this.Source = new Operation("Source");
            this.Source.IsStart = true;

            this.Sink = new Operation("Sink");
            this.Sink.IsStop = true;

            this.operations = new List<Operation>() { this.Source, this.Sink };
        }

        /*
        public void TestGraph()
        {
            List<Machine> machines = new List<Machine>();
            Machine m1 = new Machine("M1");
            Machine m2 = new Machine("M2");
            Machine m3 = new Machine("M3");
            machines.Add(m1);
            machines.Add(m2);
            machines.Add(m3);

            this.machines.AddRange(machines);

            Operation o1 = new Operation("o1", m1, 1);
            Operation o4 = new Operation("o4", m1, 3);
            Operation o7 = new Operation("o7", m1, 5);

            Operation o2 = new Operation("o2", m2, 8);
            Operation o6 = new Operation("o6", m2, 1);

            Operation o3 = new Operation("o3", m3, 8);
            Operation o5 = new Operation("o5", m3, 3);
            Operation o8 = new Operation("o8", m3, 1);

            Operation o9 = new Operation("o9", m1, 5);
            Operation o10 = new Operation("o10", m2, 2);
            Operation o11 = new Operation("o11", m3, 8);

            Operation o12 = new Operation("o12", m1, 3);
            Operation o13 = new Operation("o13", m2, 5);
            Operation o14 = new Operation("o14", m3, 7);


            Job j1 = new Job();
            j1.AddOperation(o1);
            j1.AddOperation(o2);
            j1.AddOperation(o3);

            Job j2 = new Job();
            j2.AddOperation(o4);
            j2.AddOperation(o5);

            Job j3 = new Job();
            j3.AddOperation(o6);
            j3.AddOperation(o7);
            j3.AddOperation(o8);

            Job j4 = new Job();
            j4.AddOperation(o9);
            j4.AddOperation(o10);
            j4.AddOperation(o11);

            Job j5 = new Job();
            j5.AddOperation(o12);
            j5.AddOperation(o13);
            j5.AddOperation(o14);

            Graph graph = new Graph();
            this.AddJob(j1);
            this.AddJob(j2);
            this.AddJob(j3);
            this.AddJob(j4);
            this.AddJob(j5);
          
        }
        */
        public void AddJob(Job job)
        {
            this.jobs.Add(job);
            this.Source.Succ.Add(job.GetFirstOperation());
            job.GetFirstOperation().Pred.Add(this.Source);

            this.Sink.Pred.Add(job.GetLastOperation());
            job.GetLastOperation().Succ.Add(this.Sink);

            int index = this.operations.FindIndex(x => x.Name == "Sink");
            this.operations.InsertRange(index, job.operations);

        }

        public void AddDisjunctiveEdges(Machine m)
        {
            try
            {
                bool isAcyclic = false;
                while (!isAcyclic)
                {
                    m.ShuffleOperation();
                    for (int i = 1; i < m.operations.Count; i++)
                    {
                        m.operations[i - 1].Succ.Add(m.operations[i]);
                        m.operations[i].Pred.Add(m.operations[i - 1]);

                        m.operations[i - 1].OrderInMachine = i;
                        m.operations[i].OrderInMachine = i+1;
                    }

                    isAcyclic = this.IsAcyclic();
                    if (!isAcyclic)
                    {
                        for (int i = 1; i < m.operations.Count; i++)
                        {
                            m.operations[i - 1].Succ.Remove(m.operations[i]);
                            m.operations[i].Pred.Remove(m.operations[i - 1]);
                        }
                    }


                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        /*
        public int GetMakeSpan()
        {
            return (this.operations.Find(x => x.Name == "Sink")).Let;
        }
        */

        /*
        public void ComputeCP()
        {
            this.operations = WalkListAhead(this.operations);
            this.operations = WalkListAback(this.operations);
        }
       
        private List<Operation> WalkListAhead(List<Operation> list)
        {
            int na = list.Count;
            list[0].Eet = list[0].Est + list[0].Cost;

            for (int i = 1; i < na; i++)
            {
                foreach (Operation activity in list[i].Pred)
                {
                    if (list[i].Est < activity.Eet)
                        list[i].Est = activity.Eet;
                }

                list[i].Eet = list[i].Est + list[i].Cost;
            }

            return list;
        }

        private List<Operation> WalkListAback(List<Operation> list)
        {
            int na = list.Count;
            list[na - 1].Let = list[na - 1].Eet;
            list[na - 1].Lst = list[na - 1].Let - list[na - 1].Cost;

            for (int i = na - 2; i >= 0; i--)
            {
                foreach (Operation activity in list[i].Succ)
                {
                    if (list[i].Let == 0)
                        list[i].Let = activity.Lst;
                    else
                        if (list[i].Let > activity.Lst)
                            list[i].Let = activity.Lst;
                }

                list[i].Lst = list[i].Let - list[i].Cost;
            }

            return list;
        }
        */

        

        public bool IsAcyclic()
        {
            //List<Operation> graphCopy = this.operations.ConvertAll(op => new Operation(op.Name, op.Pred, op.Succ));
            // NEEEEEEEEED TO CREATE CLONE OF THIS FUCKINGS NODES!!!

            List<Operation> graphCopy = new List<Operation>();
            graphCopy = Extensions.DeepClone(this.operations);

            List<Operation> sortedElements = new List<Operation>();
            List<Operation> nodesWithNoIncomingEdges = new List<Operation>();
            foreach (Operation vertex in graphCopy)
            {
                if (vertex.Pred.Count == 0)
                    nodesWithNoIncomingEdges.Add(vertex);
            }

            while (nodesWithNoIncomingEdges.Count > 0)
            {
                Operation node = nodesWithNoIncomingEdges[0];
                nodesWithNoIncomingEdges.RemoveAt(0);

                sortedElements.Add(node);
                //while (node.MachineNeighbors.Count > 0)
                while (node.Succ.Count > 0)
                {
                    // Node neighbour = node.MachineNeighbors[0];
                    // node.MachineNeighbors.Remove(neighbour);
                    Operation neighbour = node.Succ[0];
                    node.Succ.Remove(neighbour);
                    neighbour.Pred.Remove(node);
                    bool hasIncoming = (neighbour.Pred.Count > 0);

                    if (!hasIncoming)
                        nodesWithNoIncomingEdges.Add(neighbour);

                }
            }

            foreach (Operation vertex in graphCopy)
            {
                // if (vertex.MachineNeighbors.Count > 0)
                if (vertex.Succ.Count > 0)
                {
                    return false;
                }
            }

            return true;
        }






       // public List<Operation> GetCrititicalAcitivities(ref List<Operation> activities)
        //public void ComputeCPM(ref List<Operation> activities)
        public void ComputeCPM()    
        {
            List<Operation> activities = this.operations;
            WalkListAhead(activities.First(x => x.IsStart));
            var stopActivity = activities.First(x => x.IsStop);
            stopActivity.Let = stopActivity.Eet;
            stopActivity.Lst = stopActivity.Let - stopActivity.Cost;
            this.MakeSpan = stopActivity.Let;

            // zkontrolovat jeslti jsou vsechny nastaveny, jestli ne, tak spoustet znovu.. (check proti zacykleni)
            int maxLoopSCount = activities.Count;
            int loopCounter = 0;
            while (activities.Where(act => act.Lst == -1).ToList().Count > 0)
            {
                WalkListAback(activities.First(x => x.IsStop));
                loopCounter++;
                if (loopCounter == maxLoopSCount)
                {
                    break;
                }
            }

            this.criticalOperations = activities.Where(activity =>(activity.Eet - activity.Let == 0) && (activity.Est - activity.Lst == 0)).ToList();
            foreach (Operation op in  this.criticalOperations)
                    op.IsCritical = true;

        }
        public static void WalkListAhead(Operation position)
        {
            foreach (var successor in position.Succ)
            {
                foreach (var predecessor in successor.Pred)
                {
                    if (successor.Est < predecessor.Eet)
                        successor.Est = predecessor.Eet;
                }
                successor.Eet = successor.Est + successor.Cost;
            }

            foreach (var successor in position.Succ)
                WalkListAhead(successor);
        }

        
        public static void WalkListAback(Operation activity)
        {
            bool processAgain = true;
            // maximální počet průchodů = počet předchůdců uzlu
            int maxLoop = activity.Pred.Count;
            int loopCounter = 0;
            while (processAgain)
            {
                foreach (var predecessor in activity.Pred)
                {
                    // pokud nebyl předchůdce ještě nastaven a zároveň už neexistuje
                    // žádný následovník, který by nebyl nastaven
                    if (predecessor.Let == -1 &&
                        predecessor.Succ.Where(op => op.Lst == -1).ToList().Count == 0)
                    {
                        foreach (var successor in predecessor.Succ)
                        {
                            if (predecessor.Let == -1)
                                predecessor.Let = successor.Lst;
                            else if (predecessor.Let > successor.Lst)
                                predecessor.Let = successor.Lst;
                        }
                        predecessor.Lst = predecessor.Let - predecessor.Cost;
                    }
                }
                loopCounter++;

                // všichni předchůdci mají nastavený Lst => nezpracovávat znovu
                processAgain = !(activity.Pred.Where(op => op.Lst == -1).ToList().Count == 0);
                
                if (loopCounter == maxLoop)
                {
                    processAgain = false;
                }
            }
            foreach (var predecessor in activity.Pred)
            {
                WalkListAback(predecessor);
            }
        }

        public void InitializeGraph(ref List<Operation> opers)
        {
            foreach (Operation op in opers)
            {
                op.IsCritical = false;
                op.Est = op.Eet = 0;
                op.Lst = op.Let = -1;
            }
        }

        public void WriteCriticalPath()
        {
            //Console.WriteLine("Critical Path: ");
            System.Diagnostics.Debug.WriteLine("Critical Path: ");

            foreach (Operation activity in this.criticalOperations)
            {
                System.Diagnostics.Debug.WriteLine(" {0}", activity.ToString());
                        
                //Console.WriteLine(" {0}", activity.ToString());
            }

            System.Diagnostics.Debug.WriteLine("\n\nTotal duration: {0}\n\n", this.MakeSpan);
            //Console.Write("\n\nTotal duration: {0}\n\n", this.MakeSpan);
        }
        public void WriteGraph()
        {
            System.Diagnostics.Debug.WriteLine("Graph: ");

            foreach (Operation activity in this.operations)
            {
                //Console.WriteLine(activity.GetDetailInfo());
                System.Diagnostics.Debug.WriteLine(" {0}", activity.ToString());
            }

        }

        public void WriteMachineOperations(Machine machine)
        {
            System.Diagnostics.Debug.WriteLine("Machine {0}: ", machine);

            foreach (Operation activity in machine.operations)
            {
                //Console.WriteLine(activity.GetDetailInfo());
                System.Diagnostics.Debug.WriteLine(" {0}", activity.ToString());
            }
        }
        public void WriteGraphStructure()
        {
            Console.WriteLine("Graph structure: ");

            foreach (Operation activity in this.operations)
            {
                //Console.WriteLine(activity.GetDetailInfo());
                System.Diagnostics.Debug.WriteLine(activity.GetDetailInfo());
            }

        }

    }
}
