﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.Serialization.Formatters.Binary;

namespace ePlanner.JobShop
{
    [Serializable()]
    public class Operation
    {
        public long ID{ get; set; }
        public float Cost { get; set; }
        public string Name {get;set;}
        public int PlanID { get; set; }

        public Machine Machine;
        public List<Operation> Pred = new List<Operation>();
        public List<Operation> Succ = new List<Operation>();
        public float Est { get; set; }
        public float Lst { get; set; }
        public float Eet { get; set; }
        public float Let { get; set; }
        public int OrderInMachine { get; set; }

        public bool IsStart { get; set; }
        public bool IsStop { get; set; }

        public bool IsCritical { get; set; }

        //ProductInfo
        public string Setup { get; set; }
        public string CommentProduction { get; set; }
        public long QtyPlanned { get; set; }
        public long QtyDone { get; set; }

        
        // source or sink
        public Operation(string name)
        {
            this.Name = name;
            this.Cost = 0;
        }
        /*
        public Operation(string name, List<Operation> pred, List<Operation> succ)
        {
            this.Name = name;
            this.Pred = pred;
            this.Succ = succ;
        }
        */

        public Operation(long ID, string name, Machine m, float cost)
        {
            this.ID = ID;
            this.Name = name;
            this.Machine = m;
            this.Machine.AddOperation(this);
            this.Cost = cost;
            this.Est = this.Eet = 0;
            this.Lst = this.Let = -1;

            this.IsCritical = false;
        }

        public string GetDetailInfo()
        {
            string info = string.Format("{0} [{1}]", this.Name, this.Machine);
            for (int i = 0; i < this.Succ.Count; i++ )
            {
                info += string.Format("{1} {0}", this.Succ[i].Name, Environment.NewLine);
            }
            return info;
        }

        public override string ToString()
        {
            return string.Format("{0}[{1}][{6}],Est:{2},Eet:{3},Lst:{4},Let:{5}", this.Name, this.Cost, this.Est, this.Eet, this.Lst, this.Let, this.Machine);
        }

        public Job GetJobForOperation(List<Job> jobs)
        {
            Job job = jobs.Where(j => j.operations.Where(o => o.Name.Equals(this.Name)).ToList().Count > 0).FirstOrDefault();
            if (job != null)
            {
                return job;
            }

            return new Job();
        }
    }
}
