﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Reflection;
using System.Web;
using System.Web.Script.Serialization;
using System.Web.Services;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace ePlanner.Menu_items.Data
{
    public partial class Data : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                if (Request.QueryString["ShiftID"] != null)
                {
                    string shiftID = Convert.ToString(Request.QueryString["ShiftID"]);
                    hidShiftID.Value = shiftID;

                    // nejak nacist primo smenu
                    Page.ClientScript.RegisterStartupScript(
                        GetType(),
                        "loadData",
                        "loadData();",
                        true);
                }

            }
        }

        [WebMethod]
        public static string GetShift(string date)
        {
            DateTime shiftDate = Convert.ToDateTime(date);
            string sqlQuery = @"
                SELECT ID, ShiftNo
                FROM
                    Shift
                WHERE ShiftDate=@shiftDate";

            DataTable dataTable = Utility.GetItemsWithQuery(sqlQuery, new SqlParameter[]{ new SqlParameter("shiftDate",shiftDate)});

            Dictionary<string, string> shifts = dataTable.AsEnumerable().ToDictionary(row => Convert.ToString(row[1]), row => Convert.ToString(row[0]));
            
            string myJsonString = string.Empty;
            if (shifts.Count > 0)
            {
                myJsonString = (new JavaScriptSerializer()).Serialize(shifts);
            }
            else
            {
                Utility.InsertShift(shiftDate);
                myJsonString = GetShift(date);
            }
           
            return myJsonString;
        }

        [WebMethod]
        public static string getProductsForShift(string shiftId)
        {
            List<ProductInfo> products = new List<ProductInfo>();

            int idShift = Convert.ToInt32(shiftId);
            string sqlQuery = @"SELECT  ProductInfo.ID,
                    ProductInfo.PartNumber,ProductInfo.PartNumberSAP,
                    ProductInfo.LineID,
                    ProductInfo.CycleTime,
                    ProductInfo.Setup
            FROM ProductInfo JOIN [Plan] ON [Plan].ProductID = ProductInfo.ID 
            WHERE [Plan].ShiftID = @shiftId ";

            DataTable dataTable = Utility.GetItemsWithQuery(sqlQuery, new SqlParameter[] { new SqlParameter("shiftId", idShift) });
            List<Job> jobs = new List<Job>();
            List<string> productsNumbers = new List<string>();
            foreach (DataRow row in dataTable.Rows)
            {
                ProductInfo product = new ProductInfo(Convert.ToInt16(row["LineID"]))
                {
                    ID = long.Parse(Convert.ToString(row["ID"])),
                    PartNumber = Convert.ToString(row["PartNumber"]),
                    PartNumberSAP = Convert.ToString(row["PartNumberSAP"]),
                    Setup = Convert.ToString(row["Setup"]),
                    CycleTime = float.Parse(Convert.ToString(row["CycleTime"])),
                    LineID = Convert.ToInt16(row["LineID"])
                };

                if (!productsNumbers.Contains(product.PartNumber))
                {
                    Job job = new Job()
                    {
                        PartNumber = product.PartNumber,
                        PartNumberSAP = product.PartNumberSAP,
                        RelatedProducts = new List<ProductInfo>() { product },
                        NoSubProducts = 1
                    };
                    jobs.Add(job);
                    productsNumbers.Add(product.PartNumber);
                }
                else
                {
                    jobs[jobs.Count - 1].NoSubProducts++;
                    jobs[jobs.Count - 1].RelatedProducts.Add(product);
                }
            }
            string myJsonString = (new JavaScriptSerializer()).Serialize(jobs);
            return myJsonString;
            /*
            foreach (DataRow dr in dataTable.Rows)
            {
                ProductInfo product = new ProductInfo();
                product.PartNumber = Convert.ToString(dr["PartNumber"]);
                product.LineID = Convert.ToInt32(dr["LineID"]);
                product.CycleTime = Convert.ToSingle(dr["CycleTime"]);
                product.Setup = Convert.ToString(dr["Setup"]);
                products.Add(product);
            }

            string myJsonString = string.Empty;
            if (products.Count > 0)
            {
                myJsonString = (new JavaScriptSerializer()).Serialize(products);
            }
            
            return myJsonString;
             * */
        }


        [WebMethod]
        public static string getAvailableProducts(string shiftId)
        {
            List<ProductInfo> products = new List<ProductInfo>();

            int idShift = Convert.ToInt32(shiftId);
            string sqlQuery = @"SELECT ProductInfo.PartNumber,ProductInfo.PartNumberSAP,ProductInfo.ID,
                    ProductInfo.LineID,
                    ProductInfo.CycleTime,
                    ProductInfo.Setup
            FROM ProductInfo LEFT JOIN [Plan] ON [Plan].ProductID = ProductInfo.ID 
            WHERE  ProductInfo.ID NOT IN (
SELECT ProductInfo.ID  FROM ProductInfo JOIN [Plan] ON [Plan].ProductID = ProductInfo.ID 
            WHERE  [Plan].ShiftID = @shiftId)";

            DataTable dataTable = Utility.GetItemsWithQuery(sqlQuery, new SqlParameter[] { new SqlParameter("shiftId", idShift) });
            List<Job> jobs = new List<Job>();
            List<string> productsNumbers = new List<string>();
            foreach (DataRow row in dataTable.Rows)
            {
                ProductInfo product = new ProductInfo(Convert.ToInt16(row["LineID"]))
                {
                    ID = long.Parse(Convert.ToString(row["ID"])),
                    PartNumber = Convert.ToString(row["PartNumber"]),
                    PartNumberSAP = Convert.ToString(row["PartNumberSAP"]),
                    Setup = Convert.ToString(row["Setup"]),
                    CycleTime = float.Parse(Convert.ToString(row["CycleTime"])),
                    LineID = Convert.ToInt16(row["LineID"])
                };

                if (!productsNumbers.Contains(product.PartNumber))
                {
                    Job job = new Job()
                    {
                        PartNumber = product.PartNumber,
                        PartNumberSAP = product.PartNumberSAP,
                        RelatedProducts = new List<ProductInfo>() { product },
                        NoSubProducts = 1
                    };
                    jobs.Add(job);
                    productsNumbers.Add(product.PartNumber);
                }
                else
                {
                    jobs[jobs.Count - 1].NoSubProducts++;
                    jobs[jobs.Count - 1].RelatedProducts.Add(product);
                }
            }
            string myJsonString = (new JavaScriptSerializer()).Serialize(jobs);
            return myJsonString;
            /*
            foreach (DataRow dr in dataTable.Rows)
            {
                ProductInfo product = new ProductInfo();
                product.PartNumber = Convert.ToString(dr["PartNumber"]);
                product.LineID = Convert.ToInt32(dr["LineID"]);
                product.CycleTime = Convert.ToSingle(dr["CycleTime"]);
                product.Setup = Convert.ToString(dr["Setup"]);
                products.Add(product);
            }

            string myJsonString = string.Empty;
            if (products.Count > 0)
            {
                myJsonString = (new JavaScriptSerializer()).Serialize(products);
            }

            return myJsonString;
            */
        }

        [WebMethod]
        public static string AddProductToShift(string qtyPlanned, string partNumber, string shiftID)
        {
            Dictionary<string, string> p = new Dictionary<string, string>();

            List<ProductInfo> products = new List<ProductInfo>(Utility.GetSameProducts(partNumber));
            foreach (ProductInfo product in products)
            {
               // long lastOrder = Utility.OrderOfLastProductInLine(product.LineID);
                ePlanner.Plan lastProduct = Utility.GetLastPlanProduct(product.LineID, Convert.ToInt64(shiftID));
                Shift shift = new Shift(Convert.ToInt64(shiftID));
                
                string sqlQuery = @"INSERT INTO [Plan] (ProductID
                           ,QtyPlanned
                           ,QtyDone
                           ,StatusID
                           ,CommentPlanner
                           ,CommentProduction
                           ,[Order],
                            PlanBegin,
                            PlanEnd,
                            ShiftID)
                     VALUES	(@ProductID,@QtyPlanned,@QtyDone,@Status,@CommentPlanner,@CommentProduction,@Order,@PlanBegin, @PlanEnd, @ShiftID)
                ";
                SqlParameter parProductID = new SqlParameter("ProductID", product.ID);
                SqlParameter parQtyPlanned = new SqlParameter("QtyPlanned", qtyPlanned);
                SqlParameter parQtyDone = new SqlParameter("QtyDone", "0");
                SqlParameter parStatus = new SqlParameter("Status", "1");
                SqlParameter parCommentPlanner = new SqlParameter("CommentPlanner", "comment planner");
                SqlParameter parCommentProduction = new SqlParameter("CommentProduction", "comment production");
                SqlParameter parShftID = new SqlParameter("ShiftID", shiftID);

                DateTime beginTime = shift.ShiftDateTime;

                int productOrder = 0;

                if (lastProduct.ID > 0)
                {
                    productOrder = lastProduct.Order;
                    beginTime = lastProduct.PlanEnd.AddSeconds(1);
                }

                DateTime endTime = beginTime.AddSeconds(Convert.ToDouble(qtyPlanned) * product.CycleTime);
                
                SqlParameter parPlanBegin = new SqlParameter("PlanBegin", beginTime);
                SqlParameter parPlanEnd = new SqlParameter("PlanEnd", endTime);

                SqlParameter parOrder = new SqlParameter("Order", productOrder + 1);

                SqlParameter[] pars = new SqlParameter[] { parProductID, parQtyPlanned, parQtyDone, parStatus, parCommentPlanner, parCommentProduction, parOrder,parPlanBegin,parPlanEnd, parShftID };

                using (Database db = new Database())
                {
                    db.Insert(sqlQuery, pars);
                }



            }
            p.Add("test", "test2");
            string myJsonString = (new JavaScriptSerializer()).Serialize(p);
            return myJsonString;
        }

        [WebMethod]
        public static string RemoveProductFromShift(string partNumber, string shiftID)
        {
            Dictionary<string, string> p = new Dictionary<string, string>();

            List<ProductInfo> products = new List<ProductInfo>(Utility.GetSameProducts(partNumber));
            foreach (ProductInfo product in products)
            {
                string sqlQuery = @"DELETE FROM [Plan] WHERE ShiftID = @ShiftID AND ProductID = @ProductID ";
                SqlParameter parProductID = new SqlParameter("ProductID", product.ID);
                SqlParameter parShftID = new SqlParameter("ShiftID", shiftID);

                SqlParameter[] pars = new SqlParameter[] { parProductID, parShftID };

                using (Database db = new Database())
                {
                    db.Update(sqlQuery, pars);
                }

            }
            p.Add("test", "test2");
            string myJsonString = (new JavaScriptSerializer()).Serialize(p);
            return myJsonString;
        }

        [WebMethod]
        public static string GetShiftDate(string parShiftID)
        {
            if (string.IsNullOrEmpty(parShiftID))
            {
                return "";
            }
            else
            {
                long shiftID = Convert.ToInt64(parShiftID);

                Shift shift = new Shift(shiftID);

                string myJsonString = string.Empty;
                if (shift.ID != null)
                {
                    myJsonString = (new JavaScriptSerializer()).Serialize(shift);
                }

                return myJsonString;
            }
        }

    }
}