﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Reflection;
using System.Web;
using System.Web.UI.WebControls;

namespace ePlanner
{
    public static class Utility
    {
        public static Dictionary<long,List<Plan>> GetProductsForShift(long shiftID)
        {
            Dictionary<long, List<Plan>> storage = new Dictionary<long, List<Plan>>();

            DataTable dt = Utility.GetItemsWithQuery("SELECT *, ProductInfo.ID as ProductID, Pl.ID as PlanID FROM ProductInfo JOIN [Plan] as Pl ON ProductInfo.ID = Pl.ProductID WHERE Pl.ShiftID =@ShiftID ORDER BY ProductInfo.LineID, Pl.[Order]", new SqlParameter[] { new SqlParameter("ShiftID", shiftID) });
            foreach (DataRow dr in dt.Rows)
            {
                Plan plannedProduct = new Plan();
                plannedProduct.ID = Convert.ToInt32(dr["PlanID"]);
                plannedProduct.ProductID = Convert.ToInt32(dr["ProductID"]);
                plannedProduct.QtyPlanned = Convert.ToInt32(dr["QtyPlanned"]);
                plannedProduct.QtyDone = Convert.ToInt32(dr["QtyDone"]);
                plannedProduct.StatusID = Convert.ToInt32(dr["StatusID"]);
                plannedProduct.CommentPlanner = Convert.ToString(dr["CommentPlanner"]);
                plannedProduct.CommentProduction = Convert.ToString(dr["CommentProduction"]);
                plannedProduct.Order = Convert.ToInt32(dr["Order"]);

                if (dr["PlanBegin"] != DBNull.Value)
                {
                    plannedProduct.PlanBegin = Convert.ToDateTime(dr["PlanBegin"]);
                }
                if (dr["PlanEnd"] != DBNull.Value)
                {
                    plannedProduct.PlanEnd = Convert.ToDateTime(dr["PlanEnd"]);
                }

                if (dr["ProdStart"] != DBNull.Value)
                {
                    plannedProduct.ProdStart = Convert.ToSingle(dr["ProdStart"]);
                }
                if (dr["ProdEnd"] != DBNull.Value)
                {
                    plannedProduct.ProdEnd = Convert.ToSingle(dr["ProdEnd"]);
                }

                ProductInfo product = new ProductInfo();
                product.ID = Convert.ToInt32(dr["ProductID"]);
                product.PartNumber = Convert.ToString(dr["PartNumber"]);
                product.PartNumberSAP = Convert.ToString(dr["PartNumberSAP"]);
                product.LineID = Convert.ToInt32(dr["LineID"]);
                product.CycleTime = Convert.ToSingle(dr["CycleTime"]);
                product.Setup = Convert.ToString(dr["Setup"]);
                product.PrevLineID = dr["PrevLineID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["PrevLineID"]);
                product.NextLineID = dr["NextLineID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["NextLineID"]);
                product.ExtraInfoID = dr["ExtraInfoID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["ExtraInfoID"]); 

                plannedProduct.RelatedProduct = product;

                if (storage.ContainsKey(plannedProduct.RelatedProduct.LineID))
                {
                    storage[plannedProduct.RelatedProduct.LineID].Add(plannedProduct);
                }
                else
                {
                    storage.Add(plannedProduct.RelatedProduct.LineID, new List<Plan>() { plannedProduct });
                }

            }

            return storage;
        
        }

        public static List<Line> GetLinesForShift(long shiftID)
        {
            List<Line> lines = new List<Line>();
            DataTable dt = Utility.GetItemsWithQuery("SELECT ID, Name FROM 	Line l JOIN ( SELECT DISTINCT LineID FROM ProductInfo pif JOIN [Plan] as Pl ON pif.ID = Pl.ProductID WHERE Pl.ShiftID =@ShiftID) as PL ON l.ID=PL.LineID", new SqlParameter[] { new SqlParameter("ShiftID", shiftID) });
            foreach (DataRow dr in dt.Rows)
            {
                lines.Add(
                    new Line()
                    {
                        ID = Convert.ToInt32(dr["ID"]),
                        Name = Convert.ToString(dr["Name"])
                    });
            }
            return lines;
        }
                
        public static List<Plan[]> GetPlanProductsForShift(long shiftID)
        {
            List<Plan[]> storage = new List<Plan[]>();

            Dictionary<long, List<Plan>> dict = GetProductsForShift(shiftID);
            foreach(long lineID in dict.Keys)
            {
                storage.Add(dict[lineID].ToArray());
            }

            return storage;

        }


        public static List<ProductInfo> GetSameProducts(string partNumber)
        {
            List<ProductInfo> products = new List<ProductInfo>();

            DataTable dt = Utility.GetItems(Constants.TableName.ProductInfo, "PartNumber", partNumber);
            foreach (DataRow dr in dt.Rows)
            {
                ProductInfo product = new ProductInfo();

                product.ID = Convert.ToInt32(dr["ID"]);
                product.PartNumber = Convert.ToString(dr["PartNumber"]);
                product.PartNumberSAP = Convert.ToString(dr["PartNumberSAP"]);
                product.LineID = Convert.ToInt32(dr["LineID"]);
                product.CycleTime = Convert.ToSingle(dr["CycleTime"]);
                product.Setup = Convert.ToString(dr["Setup"]);
                product.PrevLineID = dr["PrevLineID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["PrevLineID"]);
                product.NextLineID = dr["NextLineID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["NextLineID"]);  
                product.ExtraInfoID = dr["ExtraInfoID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["ExtraInfoID"]); 

                products.Add(product);
            }
            return products;


        }

        public static List<Plan> GetProductsForLine(long lineID)
        {
            List<Plan> planItems = new List<Plan>();
/*
            string sql = @"
SELECT * From [Plan]
JOIN ProductInfo ON ProductID = ProductInfo.ID 
WHERE ProductInfo.LineID = @LineID";
*/

            string sql= @"
SELECT [Plan].ID as PlanID, [Plan].ProductID as ProductID From [Plan]
JOIN ProductInfo ON ProductID = ProductInfo.ID 
WHERE ProductInfo.LineID = @LineID";
            
            SqlParameter par = new SqlParameter("LineID", lineID);

            DataTable dt = Utility.GetItemsWithQuery(sql, new SqlParameter[] {par} );

            foreach (DataRow dr in dt.Rows)
            {
                Plan planItem = new Plan(Convert.ToString(dr["PlanID"]));
                planItems.Add(planItem);
            }
            return planItems;


        }

        

        public static long OrderOfLastProductInLine(long lineID)
        {
            string sql = "SELECT MAX([Order]) FROM [Plan] NATURAL JOIN ProductInfo ON [ProductID] = ProductInfo.ID WHERE ProductInfo.LineID=@LineId";
            long? maxOrder = null;
            using (Database db = new Database())
            {
                SqlParameter par = new SqlParameter("LineID", lineID);

                maxOrder = db.SelectInt(sql, new SqlParameter[] { par });
              
            }
            return maxOrder == null ? 0: Convert.ToInt32(maxOrder);

             
        }

        public static Plan GetLastPlanProduct(long lineID, long shiftID)
        {
            Plan pp = new Plan();
            string sql = "SELECT MAX([Plan].ID) FROM [Plan] JOIN ProductInfo ON [ProductID] = ProductInfo.ID WHERE ProductInfo.LineID=@LineId AND [Plan].ShiftID = @ShiftID";
            string lastID = null;
            using (Database db = new Database())
            {
                SqlParameter par = new SqlParameter("LineID", lineID);
                SqlParameter parShift = new SqlParameter("ShiftID", shiftID);

                lastID = db.SelectString(sql, new SqlParameter[] { par, parShift });

                pp = new Plan(lastID);

            }
            return pp;


        }

        public static ProductInfo GetChangeover(long lineID)
        {
            ProductInfo changeover = new ProductInfo();
            string sql = "SELECT ID FROM ProductInfo WHERE PartNumber = 'CHANGEOVER' AND LineID=@LineID ";
            using (Database db = new Database())
            {
                SqlParameter par = new SqlParameter("LineID", lineID);

                DataTable dt = db.Select(sql, new SqlParameter[] { par });
                foreach (DataRow dr in dt.Rows)
                {
                    changeover = new ProductInfo(Convert.ToString(dr["ID"]));
                }
            }
            return changeover;

        }

        public static Plan GetChangeover()
        {
            Plan changeover = new Plan();
            DataTable dt = Utility.GetItemsWithQuery("SELECT * FROM ProductInfo WHERE PartNumber ='CHANGEOVER' ");
            foreach (DataRow dr in dt.Rows)
            {
                Plan plannedProduct = new Plan();
                
                ProductInfo product = new ProductInfo();
                product.ID = Convert.ToInt32(dr["ID"]);
                product.PartNumber = Convert.ToString(dr["PartNumber"]);
                product.PartNumberSAP = Convert.ToString(dr["PartNumberSAP"]);
                product.LineID = Convert.ToInt32(dr["LineID"]);
                product.CycleTime = Convert.ToSingle(dr["CycleTime"]);
                product.Setup = Convert.ToString(dr["Setup"]);
                product.PrevLineID = dr["PrevLineID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["PrevLineID"]);
                product.NextLineID = dr["NextLineID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["NextLineID"]);
                product.ExtraInfoID = dr["ExtraInfoID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["ExtraInfoID"]);

                plannedProduct.RelatedProduct = product;

                changeover = plannedProduct;

            }
            return changeover;

        }

        public static List<Plan> getJobOperations(string partN)
        {
            List<Plan> operations = new List<Plan>();

            string sql = "@select * from ProductInfo LEFT JOIN [Plan] ON [Plan].ProductID=ProductInfo.ID WHERE PartNumber like '@PartNumber'";

            SqlParameter par = new SqlParameter("PartNumber", partN);

            DataTable dt = Utility.GetItemsWithQuery(sql, new SqlParameter[] { par });

            foreach (DataRow dr in dt.Rows)
            {
                Plan plannedProduct = new Plan();
                plannedProduct.ID = Convert.ToInt32(dr["PlanID"]);
                plannedProduct.ProductID = Convert.ToInt32(dr["ProductID"]);
                plannedProduct.QtyPlanned = Convert.ToInt32(dr["QtyPlanned"]);
                plannedProduct.QtyDone = Convert.ToInt32(dr["QtyDone"]);
                plannedProduct.StatusID = Convert.ToInt32(dr["StatusID"]);
                plannedProduct.CommentPlanner = Convert.ToString(dr["CommentPlanner"]);
                plannedProduct.CommentProduction = Convert.ToString(dr["CommentProduction"]);
                plannedProduct.Order = Convert.ToInt32(dr["Order"]);

                if (dr["PlanBegin"] != DBNull.Value)
                {
                    plannedProduct.PlanBegin = Convert.ToDateTime(dr["PlanBegin"]);
                }
                if (dr["PlanEnd"] != DBNull.Value)
                {
                    plannedProduct.PlanEnd = Convert.ToDateTime(dr["PlanEnd"]);
                }

                if (dr["ProdStart"] != DBNull.Value)
                {
                    plannedProduct.ProdStart = Convert.ToSingle(dr["ProdStart"]);
                }
                if (dr["ProdEnd"] != DBNull.Value)
                {
                    plannedProduct.ProdEnd = Convert.ToSingle(dr["ProdEnd"]);
                }

                if (dr["ProductID"] != DBNull.Value)
                {
                    ProductInfo product = new ProductInfo();
                    product.ID = Convert.ToInt32(dr["ProductID"]);
                    product.PartNumber = Convert.ToString(dr["PartNumber"]);
                    product.PartNumberSAP = Convert.ToString(dr["PartNumberSAP"]);
                    product.LineID = Convert.ToInt32(dr["LineID"]);
                    product.CycleTime = Convert.ToSingle(dr["CycleTime"]);
                    product.Setup = Convert.ToString(dr["Setup"]);
                    product.PrevLineID = dr["PrevLineID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["PrevLineID"]);
                    product.NextLineID = dr["NextLineID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["NextLineID"]);
                    product.ExtraInfoID = dr["ExtraInfoID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["ExtraInfoID"]);

                    plannedProduct.RelatedProduct = product;
                }
                operations.Add(plannedProduct);
            }
            return operations;

        }


        public static List<Plan> GetAllProductsForShift(long shiftID)
        {
            List<Plan> storage = new List<Plan>();

            DataTable dt = Utility.GetItemsWithQuery("SELECT *, ProductInfo.ID as ProductID, Pl.ID as PlanID FROM ProductInfo JOIN [Plan] as Pl ON ProductInfo.ID = Pl.ProductID WHERE Pl.ShiftID =@ShiftID ORDER BY ProductInfo.LineID, Pl.[Order]", new SqlParameter[] { new SqlParameter("ShiftID", shiftID) });
            foreach (DataRow dr in dt.Rows)
            {
                Plan plannedProduct = new Plan();
                plannedProduct.ID = Convert.ToInt32(dr["PlanID"]);
                plannedProduct.ProductID = Convert.ToInt32(dr["ProductID"]);
                plannedProduct.QtyPlanned = Convert.ToInt32(dr["QtyPlanned"]);
                plannedProduct.QtyDone = Convert.ToInt32(dr["QtyDone"]);
                plannedProduct.StatusID = Convert.ToInt32(dr["StatusID"]);
                plannedProduct.CommentPlanner = Convert.ToString(dr["CommentPlanner"]);
                plannedProduct.CommentProduction = Convert.ToString(dr["CommentProduction"]);
                plannedProduct.Order = Convert.ToInt32(dr["Order"]);

                if (dr["PlanBegin"] != DBNull.Value)
                {
                    plannedProduct.PlanBegin = Convert.ToDateTime(dr["PlanBegin"]);
                }
                if (dr["PlanEnd"] != DBNull.Value)
                {
                    plannedProduct.PlanEnd = Convert.ToDateTime(dr["PlanEnd"]);
                }

                if (dr["ProdStart"] != DBNull.Value)
                {
                    plannedProduct.ProdStart = Convert.ToSingle(dr["ProdStart"]);
                }
                if (dr["ProdEnd"] != DBNull.Value)
                {
                    plannedProduct.ProdEnd = Convert.ToSingle(dr["ProdEnd"]);
                }

                ProductInfo product = new ProductInfo();
                product.ID = Convert.ToInt32(dr["ProductID"]);
                product.PartNumber = Convert.ToString(dr["PartNumber"]);
                product.PartNumberSAP = Convert.ToString(dr["PartNumberSAP"]);
                product.LineID = Convert.ToInt32(dr["LineID"]);
                product.CycleTime = Convert.ToSingle(dr["CycleTime"]);
                product.Setup = Convert.ToString(dr["Setup"]);
                product.PrevLineID = dr["PrevLineID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["PrevLineID"]);
                product.NextLineID = dr["NextLineID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["NextLineID"]);
                product.ExtraInfoID = dr["ExtraInfoID"] == DBNull.Value ? 0 : Convert.ToInt32(dr["ExtraInfoID"]);

                plannedProduct.RelatedProduct = product;

                storage.Add(plannedProduct);

            }

            return storage;

        }

        public static DataTable GetItems(Constants.TableName dbTableName)
        {
            return GetItems(dbTableName, null, null);
        }

        public static DataTable GetItems(Constants.TableName dbTableName, string filter, string filterValue)
        {
            DataTable temp = new DataTable();
            using (Database db = new Database())
            {

                if (string.IsNullOrEmpty(filter))
                {
                    temp = db.Select(string.Format("SELECT * FROM [{0}]", dbTableName));
                }
                else
                {
                    SqlParameter par = new SqlParameter(filter, filterValue);
                    temp = db.Select(string.Format("SELECT * FROM [{0}] WHERE {1} = @{1}", dbTableName, filter), new SqlParameter[] { par });
                }
            }

            return temp;

        }

        public static DataTable GetItemsWithQuery(string query)
        {
            using (Database db = new Database())
            {
                DataTable temp = new DataTable();
                temp = db.Select(query);
                return temp;
            }
        }

        public static DataTable GetItemsWithQuery(string query, SqlParameter[] parameters)
        {
            using (Database db = new Database())
            {
                DataTable temp = new DataTable();
                temp = db.Select(query, parameters);
                return temp;
            }
        }

        public static bool AttachDataSource(GridView gvObject, Constants.TableName dbTableName)
        {
            return AttachDataSource(gvObject, dbTableName, null, null);
        }

        public static bool AttachDataSource(GridView gvObject, Constants.TableName dbTableName, string queryFilter, string queryFilterValue)
        {
            DataTable dataTable = Utility.GetItems(dbTableName, queryFilter, queryFilterValue);
            if (dataTable.Rows.Count > 0)
            {
                gvObject.DataSource = dataTable;
                gvObject.DataBind();

                return true;
            }
            return false;
        }

        public static List<T> DataTableToList<T>(this DataTable table) where T : class, new()
        {
            try
            {
                List<T> list = new List<T>();

                foreach (var row in table.AsEnumerable())
                {
                    T obj = new T();

                    foreach (var prop in obj.GetType().GetProperties())
                    {
                        try
                        {
                            PropertyInfo propertyInfo = obj.GetType().GetProperty(prop.Name);
                            propertyInfo.SetValue(obj, Convert.ChangeType(row[prop.Name], propertyInfo.PropertyType), null);
                        }
                        catch
                        {
                            continue;
                        }
                    }

                    list.Add(obj);
                }

                return list;
            }
            catch
            {
                return null;
            }
        }


        public static void InsertShift(DateTime date)
        {
            int NoOfShifts = Convert.ToInt16(Properties.Settings.Default["NoOfShifts"]);

            for (int i = 1; i <= NoOfShifts; i++)
            {
                string sqlQuery = @"
                INSERT INTO Shift
                    Values (@Date,@No)";

                SqlParameter parDate = new SqlParameter("Date", date);
                SqlParameter parNo = new SqlParameter("No", i);

                SqlParameter[] pars = new SqlParameter[] { parDate, parNo };

                using (Database db = new Database())
                {
                    db.Insert(sqlQuery, pars);
                }
            }
        }

        public static Plan[][] CopyArrayBuiltIn(Plan[][] source)
        {
            var len = source.Length;
            var dest = new Plan[len][];

            for (var x = 0; x < len; x++)
            {
                var inner = source[x];
                var ilen = inner.Length;
                var newer = new Plan[ilen];
                Array.Copy(inner, newer, ilen);
                dest[x] = newer;
            }

            return dest;
        }

    }
}